-------------------------------------------------
-- PLATFORM ID's --
global constant
        DOS32   = 1 -- ex.exe
       ,WIN32   = 2 -- exw.exe
       ,LINUX   = 3 -- exu
       ,FREEBSD = 3 -- exu?

-------------------------------------------------
-- COMMON DEFINES --
global constant
        NULL  =0
       ,FALSE =0   ,TRUE  =1
       ,OFF   =0   ,ON    =1
       ,NO    =0   ,YES   =1

-------------------------------------------------
-- STDIO CONSTANTS --
global constant
        STDIN   = 0
       ,STDOUT  = 1 -- STDOUT is NOT SCREEN, though it usually points there
       ,STDERR  = 2   ,SCREEN=2 -- STDERR is not redirectable

-------------------------------------------------
-- CHAR CONSTANTS --
global constant
        TAB   ='\t'
       ,CR    ='\r'
       ,LF    ='\n'
       ,CRLF  =CR&LF
       ,EOL   = LF
       ,EOF   = -1

-------------------------------------------------
-- SLASH is platform specific --
integer slash
if platform() <= WIN32 then slash = '\\' else slash = '/' end if
global constant SLASH = slash


-------------------------------------------------
-- C-TYPE ID's -- C types for .dll arguments and return value:
global constant
        CHAR     = #01000001
       ,UCHAR    = #02000001
       ,SHORT    = #01000002
       ,USHORT   = #02000002
       ,INT      = #01000004
       ,UINT     = #02000004
       ,FLOAT    = #03000004
       ,DOUBLE   = #03000008
       ,LPSZ     = #04000004
       ,LONG     = INT
       ,ULONG    = UINT
       ,POINTER  = ULONG
       ,BOOL     = CHAR
       ,BYTE     = UCHAR
       ,WORD     = USHORT
       ,DWORD    = ULONG
       ,PTR      = POINTER
       ,FLOAT32  = FLOAT
       ,FLOAT64  = DOUBLE
       ,VOID     = POINTER

-------------------------------------------------
-- EU-TYPE ID's -- Euphoria types for .dll arguments and return value:
global constant
        INTEGER = #06000004
       ,ATOM    = #07000004
       ,SEQUENCE= #08000004
       ,OBJECT  = #09000004

---------------------------------------------------------------------------------
-- MACHINE CODES --
global constant
        M_SOUND           = 1     -- system.e
       ,M_LINE            = 2     -- graphics.e
       ,M_PALETTE         = 3     -- graphics.e
       ,M_PIXEL           = 4     -- (obsolete)
       ,M_GRAPHICS_MODE   = 5     -- graphics.e
       ,M_CURSOR          = 6     -- console.e
       ,M_WRAP            = 7     -- console.e
       ,M_SCROLL          = 8     -- console.e
       ,M_SET_T_COLOR     = 9     -- console.e
       ,M_SET_B_COLOR     = 10    -- console.e
       ,M_POLYGON         = 11    -- graphics.e
       ,M_TEXTROWS        = 12    -- console.e
       ,M_VIDEO_CONFIG    = 13    -- console.e
       ,M_GET_MOUSE       = 14    -- graphics.e
       ,M_MOUSE_EVENTS    = 15    -- graphics.e
       ,M_ALLOC           = 16    -- mem.e
       ,M_FREE            = 17    -- mem.e
       ,M_ELLIPSE         = 18    -- graphics.e
       ,M_SEEK            = 19    -- file.e
       ,M_WHERE           = 20    -- file.e
       ,M_GET_PIXEL       = 21    -- (obsolete)
       ,M_DIR             = 22    -- file.e
       ,M_CURRENT_DIR     = 23    -- file.e
       ,M_MOUSE_POINTER   = 24    -- graphics.e
       ,M_GET_POSITION    = 25    -- console.e
       ,M_WAIT_KEY        = 26    -- console.e
       ,M_ALL_PALETTE     = 27    -- graphics.e
       ,M_GET_DISPLAY_PAGE= 28    -- graphics.e
       ,M_SET_DISPLAY_PAGE= 29    -- graphics.e
       ,M_GET_ACTIVE_PAGE = 30    -- graphics.e
       ,M_SET_ACTIVE_PAGE = 31    -- graphics.e
       ,M_ALLOC_LOW       = 32    -- mem.e
       ,M_FREE_LOW        = 33    -- mem.e
       ,M_INTERRUPT       = 34    -- mem.e
       ,M_SET_RAND        = 35    -- math.e
       ,M_USE_VESA        = 36    -- graphics.e
       ,M_CRASH_MESSAGE   = 37    -- error.e
       ,M_TICK_RATE       = 38    -- system.e
       ,M_GET_VECTOR      = 39    -- mem.e
       ,M_SET_VECTOR      = 40    -- mem.e
       ,M_LOCK_MEMORY     = 41    -- mem.e
       ,M_ALLOW_BREAK     = 42    -- system.e
       ,M_CHECK_BREAK     = 43    -- system.e
       ,M_MEM_COPY        = 44    -- obsolete
       ,M_MEM_SET         = 45    -- obsolete
       ,M_A_TO_F64        = 46    -- mem.e
       ,M_F64_TO_A        = 47    -- mem.e
       ,M_A_TO_F32        = 48    -- mem.e
       ,M_F32_TO_A        = 49    -- mem.e
       ,M_OPEN_DLL        = 50    -- dll.e
       ,M_DEFINE_C        = 51    -- dll.e
       ,M_CALL_BACK       = 52    -- dll.e
       ,M_PLATFORM        = 53    -- obsolete
       ,M_FREE_CONSOLE    = 54    -- system.e
       ,M_INSTANCE        = 55    -- dll.e      -- func(NULL) -> hInst(atom)
       ,M_DEFINE_VAR      = 56    -- dll.e
       ,M_CRASH_FILE      = 57    -- error.e
       ,M_GET_SCREEN_CHAR = 58    -- text_graphics.e
       ,M_PUT_SCREEN_CHAR = 59    -- text_graphics.e
       ,M_FLUSH           = 60    -- file.e
       ,M_LOCK_FILE       = 61    -- file.e
       ,M_UNLOCK_FILE     = 62    -- file.e
       ,M_CHDIR           = 63    -- file.e
       ,M_SLEEP           = 64    -- system.e   -- proc(int>0)


---------------------------------------------------------------------------------
-- Euphoria keywords and routines built in to ex, exw and exu

global constant KEYWORDS = {
    "if" ,"end" ,"then" ,"procedure" ,"else" ,"for" ,"return"
   ,"do" ,"elsif" ,"while" ,"type" ,"constant" ,"to" ,"and" ,"or"
   ,"exit" ,"function" ,"global" ,"by" ,"not" ,"include"
   ,"with" ,"without" ,"xor"}

global constant BUILTINS = {
    "length" ,"puts" ,"integer" ,"sequence" ,"position" ,"object"
   ,"append" ,"prepend" ,"print" ,"printf"
   ,"clear_screen" ,"floor" ,"getc" ,"gets" ,"get_key"
   ,"rand" ,"repeat" ,"atom" ,"compare" ,"find" ,"match"
   ,"time" ,"command_line" ,"open" ,"close" ,"trace" ,"getenv"
   ,"sqrt" ,"sin" ,"cos" ,"tan" ,"log" ,"system" ,"date" ,"remainder"
   ,"power" ,"machine_func" ,"machine_proc" ,"abort" ,"peek" ,"poke"
   ,"call" ,"sprintf" ,"arctan" ,"and_bits" ,"or_bits" ,"xor_bits"
   ,"not_bits" ,"pixel" ,"get_pixel" ,"mem_copy" ,"mem_set"
   ,"c_proc" ,"c_func" ,"routine_id" ,"call_proc" ,"call_func"
   ,"poke4" ,"peek4s" ,"peek4u" ,"profile" ,"equal" ,"system_exec"
   ,"platform"}
