include std/core.em
include std/mem/mem.e
include std/mem/binary.e
include std/mem/structs.e


global constant ERROR_OPEN_DLL = define_error(NULL)
global function open_dll(sequence file_name)
-- Open a .DLL file
 atom id
  id = machine_func(M_OPEN_DLL, file_name)
  if not id then
    throw_error(ERROR_OPEN_DLL
          ,"open_dll(): Failed to open "&file_name&"\n")
  end if
 return id
end function

global constant ERROR_DEFINE_C_VAR = define_error(NULL)
global function define_c_var(atom lib, sequence variable_name)
-- get the memory address where a global C variable is stored
 atom id
  id = machine_func(M_DEFINE_VAR, {lib, variable_name})
  if not id then
    throw_error(ERROR_DEFINE_C_VAR
          ,"define_c_var(): Failed to define "&variable_name&"\n")
  end if
 return id
end function


global constant ERROR_DEFINE_C_PROC = define_error(NULL)
global function define_c_proc(atom lib, sequence routine_name, sequence arg_sizes)
-- define a C function with VOID return type, or where the
-- return value will always be ignored
 atom id
  id = machine_func(M_DEFINE_C, {lib, routine_name, arg_sizes, NULL})
  if id < 0 then
    throw_error(ERROR_DEFINE_C_PROC
          ,"define_c_proc(): Failed to define "&routine_name&"\n")
  end if
 return id
end function

global constant ERROR_DEFINE_C_FUNC = define_error(NULL)
global function define_c_func(atom lib, sequence routine_name, sequence arg_sizes, atom return_type)
-- define a C function
 atom id
  id = machine_func(M_DEFINE_C, {lib, routine_name, arg_sizes, return_type})
   if id < 0 then
    throw_error(ERROR_DEFINE_C_FUNC
          ,"define_c_func(): Failed to define "&routine_name&"\n")
  end if
 return id
end function

type int_or_seq2(object x)
  if sequence(x) then
    if (length(x)=2) and
          integer(x[1]) and integer(x[2]) and
          (x[1]='+') then
     return TRUE
    end if
  elsif integer(x) then
   return TRUE
  end if
 return FALSE
end type

global constant ERROR_DEFINE_C_CALLBACK = define_error(NULL)
global function define_c_callback(int_or_seq2 x)
-- return a 32-bit call-back address for a Euphoria routine
-- id can be of the form: 
--     routine_id          - for Linux or Windows stdcall calls, 
-- or 
--     {'+', routine_id}   - for Windows cdecl calls
 atom id
  if integer(x) then id = x else id = x[2] end if
  if id < 0 then
    throw_error(ERROR_DEFINE_C_CALLBACK
          ,"define_c_callback(): Failed to get callback address")
   return id
  end if
  id = machine_func(M_CALL_BACK, x)
 return id
end function

global function instance()
-- WIN32: returns hInstance - handle to this instance of the program
-- DOS32: returns 0
 return machine_func(M_INSTANCE, NULL)
end function
