include std/core.em
include std/util/string.e

---------------------------------------------------------------------------------
-- TOKEN PARSING ROUTINES --

-- returns a list of indexes to the start of each token in s, delimited by c
global function get_token_index(string s, char c)
 sequence token_list
   token_list={}
   if length(s) != c then s &=c end if
   for i = 1 to length(s) do
      if s[i] = c then token_list &=i end if
   end for
   return token_list
end function

-- returns a list of tokens from s, delimited by c
global function tokenize(string s, char c)
 sequence token_list,retSeq
 integer start
   retSeq={}
   if length(s) then
      token_list = get_token_index(s,c)
      start=1
      for i = 1 to length(token_list) do
         retSeq &={s[start..token_list[i]-1]}
         start = token_list[i]+1
      end for
   end if
   return retSeq
end function

-- returns an inline list of tokens (string), delimited by c
global function inline(sequence tokens, char c)
 string s
   s=""
   if length(tokens) then
      s = tokens[1]
      for i = 2 to length(tokens) do
         s &= c&tokens[i]
      end for
   end if
   return s
end function

---------------------------------------------------------------------------------
-- CHAR TYPE ROUTINES --

global type digit_char(integer c)
  return ('0' <= c) and (c <= '9')
end type

global type hex_uchar(integer c)
  return ('A' <= c) and (c <= 'F')
end type

global type hex_lchar(integer c)
  return ('a' <= c) and (c <= 'f')
end type

global type hex_char(integer c)
  return digit_char(c) or hex_uchar(c) or hex_lchar(c)
end type

global type alpha_uchar(integer c)
  return ('A' <= c) and (c <= 'Z')
end type

global type alpha_lchar(integer c)
  return ('a' <= c) and (c <= 'z')
end type

global type alpha_char(integer c)
  return alpha_uchar(c) or alpha_lchar(c)
end type

global type alphanum_char(integer c)
  return alpha_char(c) or digit_char(c)
end type

global type whitespace_char(integer c)
  return find(c,{' ','\t','\r','\n'})
end type

global function identifier_char(char c)
  return alphanum_char(c) or (c = '_')
end function

---------------------------------------------------------------------------------
-- TOKEN TYPE ROUTINES --

-- global function get_qchar(seq s)
-- end function
-- 
-- global function get_qstring(seq s)
-- end function
-- 
-- global function get_hex(seq s)
-- end function
-- 
-- global function get_decimal(seq s)
-- end function
-- 
-- global function get_number(seq s)
-- end function
