include std/core.em

---------------------------------------------------------------------------------
-- CONSOLE IO ROUTINES --

global function wait_key(sequence msg) -- M_WAIT_KEY        = 26    --
-- Get the next key pressed by the user.
-- Wait until a key is pressed.
-- display a msg, if one is specified
  if length(msg) then puts(SCREEN,msg) end if
  return machine_func(M_WAIT_KEY, NULL)
end function

global procedure any_key(sequence msg)
-- discard result
-- Wait until a key is pressed.
-- display a msg, if one is specified
  if length(msg) then puts(SCREEN,msg) end if
  machine_proc(M_WAIT_KEY, 0)
end procedure


---------------------------------------------------------------------------------
-- CONSOLE TEXT ROUTINES --

-------------------------------------------------
-- TEXT AND VGA COLOR CONSTANTS --
sequence tmp
if platform() <= WIN32 then -- dos/win
  tmp = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15} -- windows color order
else -- linux/freebsd
  tmp = {0,4,2,6,1,5,3,7,8,12,10,14,9,13,11,15} -- linux color order
end if

global constant -- COLOR values -- for characters and pixels
  BLACK           = tmp[1] -- in graphics modes this is "transparent"
 ,BLUE            = tmp[2]
 ,GREEN           = tmp[3]
 ,CYAN            = tmp[4]
 ,RED             = tmp[5]
 ,MAGENTA         = tmp[6]
 ,BROWN           = tmp[7]
 ,WHITE           = tmp[8]
 ,GRAY            = tmp[9]
 ,BRIGHT_BLUE     = tmp[10]
 ,BRIGHT_GREEN    = tmp[11]
 ,BRIGHT_CYAN     = tmp[12]
 ,BRIGHT_RED      = tmp[13]
 ,BRIGHT_MAGENTA  = tmp[14]
 ,YELLOW          = tmp[15]
 ,BRIGHT_WHITE    = tmp[16]

global constant BLINKING = 16  -- add to color to get blinking text

-- Linux Color Order
--     BLACK          =0,
--     RED            =1,
--     GREEN          =2,
--     BROWN          =3,
--     BLUE           =4,
--     MAGENTA        =5,
--     CYAN           =6,
--     WHITE          =7,
--     GRAY           =8,
--     BRIGHT_RED     =9,
--     BRIGHT_GREEN   =10,
--     YELLOW         =11,
--     BRIGHT_BLUE    =12,
--     BRIGHT_MAGENTA =13,
--     BRIGHT_CYAN    =14,
--     BRIGHT_WHITE   =15

type color(integer x)
    return x >= 0 and x <= 255
end type


global procedure text_color(color c) -- M_SET_T_COLOR     = 9     --
-- set the foreground text color to c - text or graphics modes
-- add 16 to get blinking
  machine_proc(M_SET_T_COLOR, c)
end procedure

global procedure bk_color(color c) -- M_SET_B_COLOR     = 10    --
-- set the background color to c - text or graphics modes
  machine_proc(M_SET_B_COLOR, c)
end procedure

global function text_rows(integer rows) -- M_TEXTROWS        = 12    --
  return machine_func(M_TEXTROWS, rows)
end function

global procedure wrap(boolean on) -- M_WRAP            = 7     --
-- on = 1: characters will wrap at end of long line
-- on = 0: lines will be truncated
  machine_proc(M_WRAP, on)
end procedure

global procedure scroll(integer amount, positive_int top_line, positive_int bottom_line) -- M_SCROLL          = 8     --
-- scroll lines of text on screen between top_line and bottom_line
-- amount > 0: scroll text up by amount lines
-- amount < 0: scroll text down by amount lines
-- (had only the first parameter in v1.2)   
  machine_proc(M_SCROLL, {amount, top_line, bottom_line})
end procedure


---------------------------------------------------------------------------------
-- CONSOLE CURSOR ROUTINES --

global constant -- cursor styles:
        NO_CURSOR               = #2000
       ,UNDERLINE_CURSOR        = #0607
       ,THICK_UNDERLINE_CURSOR  = #0507
       ,HALF_BLOCK_CURSOR       = #0407
       ,BLOCK_CURSOR            = #0007


global procedure cursor(integer style) -- M_CURSOR          = 6     --
-- choose a cursor style
  machine_proc(M_CURSOR, style)
end procedure

global function get_position() -- M_GET_POSITION    = 25    --
-- return {line, column} of current cursor position
    return machine_func(M_GET_POSITION, 0)
end function

