include std/core.em


global constant PI = 3.141592653589793238
global constant PI_HALF = PI/2.0
global constant RADIANS  = PI/180


global procedure set_rand(integer seed)
-- Reset the random number generator.
-- A given value of seed will cause the same series of
-- random numbers to be generated from the rand() function
    machine_proc(M_SET_RAND, seed)
end procedure



global function iff(integer test, object isTrue, object isFalse)
  if test then return isTrue end if
  return isFalse
end function

global function in_range(object x, object min, object max)
  return (min <= x) and (x <= max)
end function


global function is_odd(object x)
  return and_bits(1,x)
end function

global function is_even(object x)
  return not and_bits(1,x)
end function


global function mod(object x1, object x2)
  return remainder(x1,x2)
end function

global function abs(object x)
  if atom(x) then
    if x < 0 then x *=-1 end if
  else
    for i = 1 to length(x) do
      x[i] = abs(x[i])
    end for
  end if
  return x
end function


global function minimum(atom_array s)
 atom m
  m = 1
  for i = 1 to length(s) do
    if s[m] > s[i] then m = i end if
  end for
  return m
end function

global function maximum(atom_array s)
 atom m
  m = 1
  for i = 1 to length(s) do
    if s[m] < s[i] then m = i end if
  end for
  return m
end function


---------------------------------------------------------------------------------
-- SUMMING ROUTINES --

-- Sum (add) all the elements of a sequence
global function sum(sequence x)
 object ret
  ret = 0
  for i = 1 to length(x) do
    ret += x[i]
  end for
  return ret
end function

-- Subtract all the elements of a sequence
global function substract(sequence x)
 object ret
  ret = 0
  for i = 1 to length(x) do
    ret -= x[i]
  end for
  return ret
end function

-- Multiply all the elements of a sequence
global function multiply(sequence x)
 object ret
  if not length(x) then return 0 end if
  ret = x[1]
  for i = 2 to length(x) do
    ret *= x[i]
  end for
  return ret
end function

-- Divide all the elements of a sequence
global function divide(sequence x)
 object ret
  if not length(x) then return 0 end if
  ret = x[1]
  for i = 2 to length(x) do
    ret /= x[i]
  end for
  return ret
end function


---------------------------------------------------------------------------------
-- TRIGONOMETRY --

global type scalar(object x)
--  values passed to arccos and arcsin must be [-1,+1]
  if atom(x) then
  	return in_range(x,-1,1)
  else
	  for i = 1 to length(x) do
	    if not scalar(x[i]) then return FALSE end if
	  end for
	  return 1
  end if
end type

-- trig formulas provided by Larry Gregg
global function arccos(scalar x)
  --  returns angle in radians
  return PI_HALF - 2 * arctan(x / (1.0 + sqrt(1.0 - x * x)))
end function

global function arcsin(scalar x)
  --  returns angle in radians
  return 2 * arctan(x / (1.0 + sqrt(1.0 - x * x)))
end function


---------------------------------------------------------------------------------
--hypotenuese
global function hypot(atom a, atom b)
  return sqrt((a*a) + (b*b))
end function


---------------------------------------------------------------------------------
-- CONVERSION ROUTINES --

-- DEG2RAD -- PI/180 * DEG
global function deg2rad(atom d)
   return d * RADIANS
end function

-- RAD2DEG
global function rad2deg(atom r)
   return r / RADIANS
end function

-- converts radians to cartesian coordinates
-- X = cos(Radian) * Radius
-- Y = sin(Radian) * Radius
global function rad2cart(atom radians, atom radius)
   return {cos(radians),sin(radians)}*radius
end function

-- converts cartesian coordinates to radians
-- X = Radius * cos(Radian)
-- X / Radius = cos(Radian)
-- arccos(X / Radius) = Radian
--
-- Radius = sqrt(x*x + y*y) -- hypoteneuse
global function cart2rad(atom x, atom y)
   return arccos(x/hypot(x,y))
end function


---------------------------------------------------------------------------------
-- PHYSICS NOTES --

-- Earth Gravity = 9.8 m/s^2
-- Moon Gravity = 1/6 * Earth Gravity


-- Avg Velocity = Displacement / Time
-- Avg Acceleration = Velocity Change / Time
-- Displacement = (Vi + Vf)*t /2
-- Vf = Vi + a*t
-- d = Vi*t + (a*t^2)/2
-- Vf^2 = Vi^2 + 2*a*d

-- 1 Newton = 1kg * 1m/s
-- Force = Mass * Acceleration

-- Weight = Mass*Gravity
-- Normal Force = -(Mass*-Gravity)
-- Friction Force = Friction*Normal Force
