include std/core.em

---------------------------------------------------------------------------------
-- STRING.E --

global function lower(object x)
-- convert atom or sequence to lower case
    return x + (('A' <= x and x <= 'Z') * ' ')
end function

global function upper(object x)
-- convert atom or sequence to upper case
    return x - (('a' <= x and x <= 'z') * ' ')
end function


---------------------------------------------------------------------------------
-- WHITESPACE ROUTINES --

sequence WhiteSpace         WhiteSpace = {" \t\r"} -- default whitespace chars

-- retrieve the whitespace char set that is currently being used
global function get_whitespace()
  return WhiteSpace
end function

-- define what chars are considered to be whitespace
global procedure set_whitespace(sequence white)
  WhiteSpace = white
end procedure

-- trim all whitespace from the start of a string
global function ltrim_white(string s)
 integer inc
  inc = 1
  while inc <= length(s) do
    if find(s[inc],WhiteSpace) then inc +=1 else exit end if
  end while
  return s[inc..length(s)]
end function

--trim all whitespace from the end of a string
global function rtrim_white(string s)
 integer inc
  inc = length(s)
  while inc >= 1 do
    if find(s[inc],WhiteSpace) then inc -=1 else exit end if
  end while
  return s[1..inc]
end function

-- trim whitespace from both ends of a string
global function trim_white(string s)
   return ltrim_white(rtrim_white(s))
end function

-- trim excess spaces from a string. All consecutive space will be reduced to 1 space only
-- IE. "  This          is a  test", becomes " This is a test"
global function compress_white(string s)
 sequence retSeq
 integer found
   retSeq =""
   found = 0
   for i=1 to length(s) do
      if find(s[i],WhiteSpace) then found +=1 else found = 0 end if
      if found <= 1 then retSeq &=s[i] end if
   end for
   return retSeq
end function


---------------------------------------------------------------------------------
global function sprint(object x)
-- Return the string representation of any Euphoria data object. 
-- This is the same as the output from print(1, x) or '?', but it's
-- returned as a string sequence rather than printed.
 sequence s
   if atom(x) then
    return sprintf("%.10g", x)
   else
      s = "{"
      for i = 1 to length(x) do
         s &= sprint(x[i])
         if i < length(x) then  s &= ','  end if
      end for
      s &= "}"
      return s
   end if
end function
